package envoy.client.ui;

import java.awt.*;
import java.awt.TrayIcon.MessageType;

import javafx.application.Platform;
import javafx.stage.Stage;

import envoy.client.event.MessageCreationEvent;
import envoy.data.Message;
import envoy.event.EventBus;
import envoy.exception.EnvoyException;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>StatusTrayIcon.java</strong><br>
 * Created: <strong>3 Dec 2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy Client v0.2-alpha
 */
public class StatusTrayIcon {

	/**
	 * The {@link TrayIcon} provided by the System Tray API for controlling the
	 * system tray. This includes displaying the icon, but also creating
	 * notifications when new messages are received.
	 */
	private final TrayIcon trayIcon;

	/**
	 * A received {@link Message} is only displayed as a system tray notification if
	 * this variable is set to {@code true}.
	 */
	private boolean displayMessages = false;

	/**
	 * Creates a {@link StatusTrayIcon} with the Envoy logo, a tool tip and a pop-up
	 * menu.
	 *
	 * @param stage the stage which focus determines if message
	 *              notifications are displayed
	 * @throws EnvoyException if the currently used OS does not support the System
	 *                        Tray API
	 * @since Envoy Client v0.2-beta
	 */
	public StatusTrayIcon(Stage stage) throws EnvoyException {
		if (!SystemTray.isSupported()) throw new EnvoyException("The Envoy tray icon is not supported.");

		trayIcon = new TrayIcon(IconUtil.loadAWTCompatible("/icons/envoy_logo.png"), "Envoy");
		trayIcon.setImageAutoSize(true);
		trayIcon.setToolTip("You are notified if you have unread messages.");

		final PopupMenu popup = new PopupMenu();

		final MenuItem exitMenuItem = new MenuItem("Exit");
		exitMenuItem.addActionListener(evt -> System.exit(0));
		popup.add(exitMenuItem);

		trayIcon.setPopupMenu(popup);

		// Only display messages if the stage is not focused
		stage.focusedProperty().addListener((ov, onHidden, onShown) -> displayMessages = ov.getValue());

		// Show the window if the user clicks on the icon
		trayIcon.addActionListener(evt -> Platform.runLater(() -> { stage.show(); stage.requestFocus(); }));

		// Start processing message events
		// TODO: Handle other message types
		EventBus.getInstance()
			.register(MessageCreationEvent.class,
					evt -> { if (displayMessages) trayIcon.displayMessage("New message received", evt.get().getText(), MessageType.INFO); });
	}

	/**
	 * Makes this {@link StatusTrayIcon} appear in the system tray.
	 *
	 * @throws EnvoyException if the status icon could not be attaches to the system
	 *                        tray for system-internal reasons
	 * @since Envoy Client v0.2-alpha
	 */
	public void show() throws EnvoyException {
		try {
			SystemTray.getSystemTray().add(trayIcon);
		} catch (final AWTException e) {
			throw new EnvoyException("Could not attach Envoy tray icon to system tray.", e);
		}
	}
}
